@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1995-2002 Silicon Graphics, Inc.  All Rights Reserved.
- * Copyright (c) 2013 Red Hat, Inc.
+ * Copyright (c) 2013-2017 Red Hat, Inc.
  * 
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -14,9 +14,31 @@
  */
 
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
 #include <stddef.h>
 
+void
+__pmHashInit(__pmHashCtl *hcp)
+{
+    memset(hcp, 0, sizeof(*hcp));
+    /* NB: some __pmHash* clients rely on this to eschew explicit
+       initialization for .bss / .data-resident __pmHashCtl structs. */
+}
+
+/*
+ * Used to preallocate the hash table when the size is known ahead of time.
+ * This avoids the overhead of growing and relinking the hash chains.
+ */
+int
+__pmHashPreAlloc(int hsize, __pmHashCtl *hcp)
+{
+    if ((hcp->hash = (__pmHashNode **)calloc(hsize, sizeof(__pmHashNode *))) == NULL)
+	return -oserror();
+
+    hcp->hsize = hsize;
+    return 0; /* ok */
+}
+
 __pmHashNode *
 __pmHashSearch(unsigned int key, __pmHashCtl *hcp)
 {
@@ -104,6 +126,7 @@ __pmHashDel(unsigned int key, void *data, __pmHashCtl *hcp)
 	    else
 		lhp->next = hp->next;
 	    free(hp);
+	    hcp->nodes--;
 	    return 1;
 	}
 	lhp = hp;
@@ -112,6 +135,16 @@ __pmHashDel(unsigned int key, void *data, __pmHashCtl *hcp)
     return 0;
 }
 
+void
+__pmHashClear(__pmHashCtl *hcp)
+{
+    if (hcp->hsize != 0) {
+	free(hcp->hash);
+	hcp->hash = NULL;
+	hcp->hsize = 0;
+    }
+}
+
 /*
  * Iterate over the entire hash table.  For each entry, call *cb,
  * passing *cdata and the current key/value pair.  The function's
@@ -185,3 +218,32 @@ __pmHashWalk(__pmHashCtl *hcp, __pmHashWalkState state)
     hcp->next = node->next;
     return node;
 }
+
+/*
+ * free __pmHashNode's and reset the hash table to empty
+ * ... it is assumed the caller has already walked the hash lists
+ * and freed anything hanging off hp->data before calling
+ * __pmHashFree()
+ */
+void
+__pmHashFree(__pmHashCtl *hcp)
+{
+    __pmHashNode	*hp;
+    __pmHashNode	*lhp = NULL;
+    int			i;
+
+    if (hcp->hsize == 0)
+	return;
+
+    for (i = 0; i < hcp->hsize; i++) {
+	for (hp = hcp->hash[i]; hp != NULL; hp = hp->next) {
+	    if (lhp != NULL)
+		free(lhp);
+	    lhp = hp;
+	}
+    }
+    if (lhp != NULL)
+	free(lhp);
+
+    __pmHashClear(hcp);
+}
