@@ -173,6 +173,7 @@ enum {
     _PM_NETSTAT_TCPEXT_TCPMIGRATEREQSUCCESS,
     _PM_NETSTAT_TCPEXT_TCPMIGRATEREQFAILURE,
     _PM_NETSTAT_TCPEXT_TCPLOSS,
+    _PM_NETSTAT_TCPEXT_TCPPLBREHASH,
 
     _PM_NETSTAT_TCPEXT_NFIELDS /* must be last */
 };
@@ -202,6 +203,7 @@ enum {
     _PM_NETSTAT_MPTCPEXT_NODSSINWINDOW,
     _PM_NETSTAT_MPTCPEXT_DUPLICATEDATA,
     _PM_NETSTAT_MPTCPEXT_ADDADDR,
+    _PM_NETSTAT_MPTCPEXT_ADDADDRDROPS,
     _PM_NETSTAT_MPTCPEXT_ECHOADD,
     _PM_NETSTAT_MPTCPEXT_PORTADD,
     _PM_NETSTAT_MPTCPEXT_MPJOINPORTSYNRX,
@@ -210,10 +212,24 @@ enum {
     _PM_NETSTAT_MPTCPEXT_MISMATCHPORTSYNRX,
     _PM_NETSTAT_MPTCPEXT_MISMATCHPORTACKRX,
     _PM_NETSTAT_MPTCPEXT_RMADDR,
+    _PM_NETSTAT_MPTCPEXT_RMADDRDROPS,
     _PM_NETSTAT_MPTCPEXT_RMSUBFLOW,
     _PM_NETSTAT_MPTCPEXT_MPPRIOTX,
     _PM_NETSTAT_MPTCPEXT_MPPRIORX,
     _PM_NETSTAT_MPTCPEXT_RCVPRUNED,
+    _PM_NETSTAT_MPTCPEXT_MPFAILTX,
+    _PM_NETSTAT_MPTCPEXT_MPFAILRX,
+    _PM_NETSTAT_MPTCPEXT_SUBFLOWSTALE,
+    _PM_NETSTAT_MPTCPEXT_SUBFLOWRECOVER,
+    _PM_NETSTAT_MPTCPEXT_MPFASTCLOSETX,
+    _PM_NETSTAT_MPTCPEXT_MPFASTCLOSERX,
+    _PM_NETSTAT_MPTCPEXT_MPRSTTX,
+    _PM_NETSTAT_MPTCPEXT_MPRSTRX,
+    _PM_NETSTAT_MPTCPEXT_INFINITEMAPTX,
+    _PM_NETSTAT_MPTCPEXT_SNDWNDSHARED,
+    _PM_NETSTAT_MPTCPEXT_RCVWNDSHARED,
+    _PM_NETSTAT_MPTCPEXT_RCVWNDCONFLICTUPDATE,
+    _PM_NETSTAT_MPTCPEXT_RCVWNDCONFLICT,
 
     _PM_NETSTAT_MPTCPEXT_NFIELDS /* must be last */
 };
