@@ -54,9 +54,12 @@ _cleanup()
 {
     $USE_SYSLOG && [ $status -ne 0 ] && \
     $PCP_SYSLOG_PROG -p daemon.error "$prog failed - see $PROGLOG"
-    [ -s "$PROGLOG" ] || rm -f "$PROGLOG"
+    if [ "$PROGLOG" != "/dev/tty" ]
+    then
+	[ -s "$PROGLOG" ] || rm -f "$PROGLOG"
+    fi
     lockfile=`cat $tmp/lock 2>/dev/null`
-    rm -f "$lockfile"
+    [ -n "$lockfile" ] && rm -f "$lockfile"
     rm -rf $tmp
 }
 trap "_cleanup; exit \$status" 0 1 2 3 15
@@ -123,7 +126,7 @@ Options:
   -V,--verbose            verbose output (multiple times for very verbose)
   -x=N,--compress-after=N  compress pmie log files after N days
   -X=PROGRAM,--compressor=PROGRAM  use PROGRAM for pmie log file compression
-  -Y=REGEX,--regex=REGEX  egrep filter when compressing files ["$COMPRESSREGEX"]
+  -Y=REGEX,--regex=REGEX  grep -E filter when compressing files ["$COMPRESSREGEX"]
   --help
 EOF
 
@@ -216,9 +219,16 @@ done
 if $SHOWME
 then
     :
+elif [ "$PROGLOG" = "/dev/tty" ]
+then
+    # special case for debugging ... no salt away previous, no chown, no exec
+    #
+    :
 else
     # Salt away previous log, if any ...
     #
+    PROGLOGDIR=`dirname "$PROGLOG"`
+    [ -d "$PROGLOGDIR" ] || mkdir_and_chown "$PROGLOGDIR" 755 $PCP_USER:$PCP_GROUP 2>/dev/null
     _save_prev_file "$PROGLOG"
     # After argument checking, everything must be logged to ensure no mail is
     # accidentally sent from cron.  Close stdout and stderr, then open stdout
@@ -239,6 +249,8 @@ then
     exit
 fi
 
+# use yesterday's datestamp
+#
 SUMMARY_LOGNAME=`pmdate -1d %Y%m%d`
 
 _error()
@@ -321,7 +333,8 @@ _parse_control()
     controlfile="$1"
     line=0
 
-    if echo "$controlfile" | grep -q -e '\.rpmsave' -e '\.rpmnew'
+    if echo "$controlfile" | grep -q -e '\.rpmsave$' -e '\.rpmnew$' -e '\.rpmorig$' \
+	-e '\.dpkg-dist$' -e '\.dpkg-old$' -e '\.dpkg-new$' >/dev/null 2>&1
     then
 	echo "Warning: ignored backup control file \"$controlfile\""
 	return
@@ -522,15 +535,21 @@ NR == 3	{ printf "p_pmcd_host=\"%s\"\n", $0; next }
 	    # now move current logfile name aside and SIGHUP to "roll the logs"
 	    # creating a new logfile with the old name in the process.
 	    #
-	    $SHOWME && echo "+ mv $logfile ${logfile}.{SUMMARY_LOGNAME}"
-	    if mv $logfile ${logfile}.${SUMMARY_LOGNAME}
+	    if $SHOWME
 	    then
-		$VERY_VERBOSE && echo "+ $KILL -s HUP $pid"
-		eval $KILL -s HUP $pid
-		echo ${logfile}.${SUMMARY_LOGNAME} >> $tmp/mail
+		echo "+ cat $logfile >>$logfile.$SUMMARY_LOGNAME"
 	    else
-		_error "problems moving logfile \"$logfile\" for host \"$host\""
-		touch $tmp/err
+		echo "---- from $prog @ `date` ----" >>$logfile.$SUMMARY_LOGNAME
+		if cat $logfile >>$logfile.$SUMMARY_LOGNAME
+		then
+		    $VERY_VERBOSE && echo "+ $KILL -s HUP $pid"
+		    eval $KILL -s HUP $pid
+		    echo $logfile.$SUMMARY_LOGNAME >> $tmp/mail
+		else
+		    _error "problems moving logfile \"$logfile\" for host \"$host\""
+		    touch $tmp/err
+		fi
+		chown $PCP_USER:$PCP_GROUP $logfile.$SUMMARY_LOGNAME >/dev/null 2>&1
 	    fi
 	fi
 
@@ -560,7 +579,7 @@ NR == 3	{ printf "p_pmcd_host=\"%s\"\n", $0; next }
 	#
 	if [ ! -z "$COMPRESSAFTER" ]
 	then
-	    _date_filter $COMPRESSAFTER | egrep -v "$COMPRESSREGEX" >$tmp/list
+	    _date_filter $COMPRESSAFTER | grep -E -v "$COMPRESSREGEX" >$tmp/list
 	    if [ -s $tmp/list ]
 	    then
 		if $VERBOSE
@@ -595,17 +614,17 @@ then
     do
 	[ -f $logfile ] && logs="$logs $logfile"
     done
-    egrep -v '( OK | OK$|^$|^Log |^pmie: PID)' $logs > $tmp/logmail
+    grep -E -v '( OK | OK$|^$|^Log |^pmie: PID)' $logs > $tmp/logmail
     if [ ! -s "$tmp/logmail" ]
     then
 	:
     elif [ ! -z "$MAIL" ]
     then
-	egrep -v '( OK | OK$|^$)' $logs | \
+	grep -E -v '( OK | OK$|^$)' $logs | \
 	    $MAIL -s "PMIE summary for $LOCALHOSTNAME" $MAILME
     else
 	echo "$prog: PMIE summary for $LOCALHOSTNAME ..."
-	egrep -v '( OK | OK$|^$)' $logs
+	grep -E -v '( OK | OK$|^$)' $logs
     fi
     rm -f $tmp/mail $tmp/logmail
 fi
