@@ -21,6 +21,7 @@
  *	APPL0	- lexical scanning
  *	APPL1	- parse/expression tree construction
  *	APPL2	- expression execution
+ *	APPL3	- status file operations
  */
 
 #include <ctype.h>
@@ -48,6 +49,8 @@ static char *intro  = "Performance Co-Pilot Inference Engine (pmie), "
 		      "Version %s\n\n%s%s";
 char	*clientid;
 
+int	runfromcontrol;
+
 static FILE *logfp;
 static char logfile[MAXPATHLEN];
 static char perffile[MAXPATHLEN];	/* /var/tmp/<pid> file name */
@@ -101,6 +104,7 @@ static pmLongOptions longopts[] = {
     { "", 0, 'H', NULL }, /* was: no DNS lookup on the default hostname */
     { "", 1, 'j', "FILE", "stomp protocol (JMS) file" },
     { "logfile", 1, 'l', "FILE", "send status and error messages to FILE" },
+    { "note", 1, 'm', "MSG", "descriptive note" },
     { "username", 1, 'U', "USER", "run as named USER in daemon mode [default pcp]" },
     PMAPI_OPTIONS_HEADER("Reporting options"),
     { "buffer", 0, 'b', 0, "one line buffered output stream, stdout on stderr" },
@@ -116,7 +120,7 @@ static pmLongOptions longopts[] = {
 
 static pmOptions opts = {
     .flags = PM_OPTFLAG_STDOUT_TZ,
-    .short_options = "a:A:bc:CdD:efFHh:j:l:n:O:PqS:t:T:U:vVWXxzZ:?",
+    .short_options = "a:A:bc:CdD:efFHh:j:l:m:n:O:PqS:t:T:U:vVWXxzZ:?",
     .long_options = longopts,
     .short_usage = "[options] [filename ...]",
     .override = override,
@@ -342,6 +346,11 @@ startmonitor(void)
     atexit(stopmonitor);
 
     /* create and initialize memory mapped performance data file */
+    if (pmDebugOptions.appl3) {
+	pmiestats_t	ps;
+	fprintf(stderr, "pmiestats_t: %d bytes, version offset: %d\n",
+	    (int)sizeof(pmiestats_t), (int)((char *)&ps.version - (char *)&ps));
+    }
     pmsprintf(perffile, sizeof(perffile),
 		"%s%c%" FMT_PID, pmie_dir, pmPathSeparator(), (pid_t)getpid());
     unlink(perffile);
@@ -447,6 +456,14 @@ sighupproc(int sig)
    dorotate = 1;
 }
 
+static void
+sigusr1(int sig)
+{
+    __pmSetSignalHandler(SIGUSR1, sigusr1);
+    pmNotifyErr(LOG_INFO, "%s caught SIGUSR1", pmGetProgname());
+    dotaskdump = 1;
+}
+
 static void
 sigbadproc(int sig)
 {
@@ -590,6 +607,10 @@ getargs(int argc, char *argv[])
 	    isdaemon = 1;
 	    break;
 
+	case 'm':			/* note, probably from pmie_check */
+	    runfromcontrol = (strcmp(opts.optarg, "pmie_check") == 0);
+	    break;
+
 	case 'U': 			/* run as named user */
 	    username = opts.optarg;
 	    isdaemon = 1;
@@ -703,6 +724,11 @@ getargs(int argc, char *argv[])
 	__pmSetSignalHandler(SIGHUP, SIG_IGN);
     }
 
+    /*
+     * always catch SIGUSR1
+     */
+    __pmSetSignalHandler(SIGUSR1, sigusr1);
+
     /*
      * -b ... force line buffering and stdout onto stderr
      */
