@@ -163,14 +163,11 @@ _pmUnpackLabelSet(__int32_t *recbuf, unsigned int *ltype, unsigned int *ident,
     tbuf = (char *)recbuf;
     k = sizeof(__pmLogHdr);
     if (type == TYPE_LABEL) {
-	memcpy((void *)&tsp->sec, (void *)&recbuf[2], sizeof(tsp->sec));
-	tsp->nsec = recbuf[4];
-	__ntohpmTimestamp(tsp);
+	__pmLoadTimestamp(&recbuf[2], tsp);
 	k += 3 * sizeof(__int32_t);
     }
     else if (type == TYPE_LABEL_V2) {
-	tsp->sec = ntohl(recbuf[2]);
-	tsp->nsec = ntohl(recbuf[3]) * 1000;
+	__pmLoadTimeval(&recbuf[2], tsp);
 	k += 2 * sizeof(__int32_t);
     }
     else {
@@ -178,7 +175,6 @@ _pmUnpackLabelSet(__int32_t *recbuf, unsigned int *ltype, unsigned int *ident,
 	exit(1);
     }
 
-
     *ltype = ntohl(*((unsigned int*)&tbuf[k]));
     k += sizeof(*ltype);
 
@@ -862,7 +858,10 @@ do_labelset(void)
 		}
 			    
 		if (pmDebugOptions.appl0) {
-		    fprintf(stderr, "Metadata: write LabelSet %s @ offset=%ld\n",
+		    fprintf(stderr, "Metadata: write ");
+		    if (outarch.version == PM_LOG_VERS02)
+			fprintf(stderr, "V2 ");
+		    fprintf(stderr, "LabelSet %s @ offset=%ld\n",
 			    __pmLabelIdentString(lp->new_id, type,
 						 idbuf, sizeof(idbuf)),
 			    out_offset);
@@ -987,7 +986,10 @@ do_labelset(void)
 	}
 
 	if (pmDebugOptions.appl0) {
-	    fprintf(stderr, "Metadata: write LabelSet %s @ offset=%ld\n",
+	    fprintf(stderr, "Metadata: write ");
+	    if (outarch.version == PM_LOG_VERS02)
+		fprintf(stderr, "V2 ");
+	    fprintf(stderr, "LabelSet %s @ offset=%ld\n",
 		    __pmLabelIdentString(ident, type, idbuf, sizeof(idbuf)),
 		    out_offset);
 	}
